﻿
Imports System.Runtime.InteropServices

Module Module_Fonts

    Private InstallableFontsFolder As String = Application.StartupPath + "\Fonts"
    Friend Sub OpenFontsFolder()
        ' -------------------------------------------------------------- prompt to user
        MessageBox.Show("You are about to open the ""Fonts"" folder." + vbCrLf + vbCrLf + _
                        "In this folder you will find: " + vbCrLf + _
                        " - A link to the System Fonts folder" + vbCrLf + _
                        "    useful to disinstall (delete) the fonts." + vbCrLf + _
                        " - A link to an Internet site to download free fonts." + vbCrLf + _
                        " - Some installable fonts." + vbCrLf + vbCrLf + _
                        "Warning: few types of fonts can be used:" + vbCrLf + _
                        " 1) They must be ""Monospaced""" + vbCrLf + _
                        " 2) They must align ""Tabs"" exactly as the ""Courier New"" font." + vbCrLf + vbCrLf + _
                        "The fonts list will show only the valid fonts" + vbCrLf + _
                        " otherwise all the programs alignments would be lost." + vbCrLf + _
                        "", _
                        "Message from Automation", MessageBoxButtons.OK)
        Process.Start(InstallableFontsFolder)
    End Sub

    Friend Function GetValidFontName_Monospaced(ByVal FontName As String) As String
        Dim fn() As String = GetInstalledFontNames_Monospaced()
        If Array.IndexOf(fn, FontName) < 0 Then
            Return "Courier New"
        End If
        Return FontName
    End Function

    Friend Function GetInstalledFontNames() As String()
        Dim s(-1) As String
        Dim installedFonts As New Drawing.Text.InstalledFontCollection
        Dim fontFamilies() As FontFamily = installedFonts.Families()
        For Each ff As FontFamily In fontFamilies
            ReDim Preserve s(s.Length)
            s(s.Length - 1) = ff.Name
        Next
        Return s
    End Function

    Friend Function GetInstalledFontNames_Monospaced() As String()
        ' -----------------------------------------------------------------------------
        'Dim sw As Stopwatch = New Stopwatch
        'sw.Start()
        ' -----------------------------------------------------------------------------
        Dim AllFontNames() As String = GetInstalledFontNames()
        Dim s(-1) As String
        For Each fn As String In AllFontNames
            Try
                Dim ft As Font = New Font(fn, 15)
                If IsFontMonospacedAndSimilarToCurierNew(ft) Then
                    ReDim Preserve s(s.Length)
                    s(s.Length - 1) = fn
                End If
            Catch
            End Try
        Next
        ' -----------------------------------------------------------------------------
        'Form1.Text = "Found " + s.Length.ToString + " monospaced fonts / Scanned " + _
        '             AllFontNames.Length.ToString + " fontfamilies / Total time = " + _
        '             sw.Elapsed.TotalMilliseconds.ToString("0") + " mS"
        ' -----------------------------------------------------------------------------
        Return s
    End Function

    Private Function IsFontInstalled(ByVal FontFileName As String) As Boolean
        Dim InstalledFontNames() As String = GetInstalledFontNames()
        For Each fn As String In InstalledFontNames
            If fn.Replace(" ", "").ToLower.StartsWith(FontFileName.Replace(" ", "").ToLower) Then
                Return True
            End If
        Next
        Return False
    End Function

    Private Function IsFontMonospaced(ByVal ft As Font) As Boolean
        Dim TestGraphics As Graphics = Graphics.FromImage(New Bitmap(64, 64))
        Dim TestChars() As Char = New Char() {"."c, ","c, "i"c, "a"c, "Z"c, "%"c, "#"c, "a"c, "l"c, "m"c}
        Dim charWidth As Single = TestGraphics.MeasureString("B", ft).Width
        Dim monospaced As Boolean = True
        For Each c As Char In TestChars
            If TestGraphics.MeasureString(c.ToString, ft).Width <> charWidth Then
                Return False
            End If
        Next
        Return monospaced
    End Function

    Private Function IsFontMonospacedAndSimilarToCurierNew(ByVal ft As Font) As Boolean
        Dim TestGraphics As Graphics = Graphics.FromImage(New Bitmap(500, 64))
        Dim str1 As String = "Key Left 		......"
        Dim str2 As String = "Key Up 		GGGGGG"
        Dim str3 As String = "sss			iiiiii"
        Dim charWidth As Single = TestGraphics.MeasureString(str1, ft).Width
        Dim monospaced As Boolean = True
        If TestGraphics.MeasureString(str2, ft).Width <> charWidth Then
            Return False
        End If
        If TestGraphics.MeasureString(str3, ft).Width <> charWidth Then
            Return False
        End If
        Return monospaced
    End Function

    Private Function GetFontNamesFromFolder(ByVal folder As String, _
                                            ByVal WithExtension As Boolean) As String()
        Dim sa(-1) As String
        IO.Directory.CreateDirectory(folder)
        For Each f As String In My.Computer.FileSystem.GetFiles(InstallableFontsFolder, _
                                                                FileIO.SearchOption.SearchAllSubDirectories)
            If f.ToLower.EndsWith(".ttf") Or f.ToLower.EndsWith(".otf") Then
                If WithExtension Then
                    f = IO.Path.GetFileName(f)
                Else
                    f = IO.Path.GetFileNameWithoutExtension(f)
                End If
                ReDim Preserve sa(sa.Length)
                sa(sa.Length - 1) = f
            End If
        Next
        Return sa
    End Function

    Private Function GetFontNamesFromFolder_WithCompletePath(ByVal folder As String) As String()
        Dim sa(-1) As String
        IO.Directory.CreateDirectory(folder)
        For Each f As String In My.Computer.FileSystem.GetFiles(InstallableFontsFolder, _
                                                                FileIO.SearchOption.SearchAllSubDirectories)
            If f.ToLower.EndsWith(".ttf") Or f.ToLower.EndsWith(".otf") Then
                ReDim Preserve sa(sa.Length)
                sa(sa.Length - 1) = f
            End If
        Next
        Return sa
    End Function


    ' ===================================================================================================
    '   INSTALL FONTS
    ' ===================================================================================================
    Friend Sub InstallNewFonts()
        ' ------------------------------------------------------------------ installed fonts without spaces
        Dim installed() As String = GetInstalledFontNames_Monospaced()
        For i As Int32 = 0 To installed.Length - 1
            installed(i) = installed(i).Replace(" ", "").ToLower
        Next
        ' ------------------------------------------------------------------ font folder file names
        Dim sa() As String = GetFontNamesFromFolder_WithCompletePath(InstallableFontsFolder)
        Dim NewFilesCount As Int32 = 0
        For Each s As String In sa
            If Not IsFontInstalled(IO.Path.GetFileNameWithoutExtension(s)) Then
                NewFilesCount += 1
            End If
        Next
        ' ------------------------------------------------------------------ test font files
        If NewFilesCount > 0 Then
            ' -------------------------------------------------------------- prompt to user
            MessageBox.Show("There are " + NewFilesCount.ToString + " new fonts to install." + vbCrLf + vbCrLf + _
                            "To install the new fonts: " + vbCrLf + _
                            " 1) Press the ""OK"" button." + vbCrLf + _
                            " 2) Press the ""Install"" button on the upper left of each font." + vbCrLf + _
                            " 3) Close and reopen the Automation application." + vbCrLf, _
                            "Message from Automation", MessageBoxButtons.OK)
            ' -------------------------------------------------------------- open file names to install
            For Each s As String In sa
                If Not IsFontInstalled(IO.Path.GetFileNameWithoutExtension(s)) Then
                    Process.Start(s)
                End If
            Next
        Else
            MessageBox.Show("All the fonts are already installed.", _
                            "Message from Automation", MessageBoxButtons.OK)
        End If
    End Sub


    ' ---------------------------------------------------
    ' No permissions to copy to the "Fonts" folder
    ' ---------------------------------------------------
    'Private Sub CopyFontToSystemFonFolder(ByVal FontFile As String)
    '    Dim SystemDir As String = Environment.GetFolderPath(Environment.SpecialFolder.System)
    '    Dim FontsDir As String = IO.Directory.GetParent(SystemDir).FullName + "\Fonts"
    '    Dim fi As IO.FileInfo = New IO.FileInfo(FontFile)
    '    Dim fiTarget As IO.FileInfo = New IO.FileInfo(FontsDir + "\" + fi.Name)
    '    If Not fiTarget.Exists Then
    '        fi.CopyTo(FontsDir + "\" + fi.Name)
    '    End If
    'End Sub

    '<DllImport("gdi32")> _
    'Public Function AddFontResource(ByVal lpFileName As String) As Integer
    'End Function

    '<DllImport("user32.dll")> _
    'Public Function SendMessage(ByVal hWnd As Integer, ByVal Msg As UInteger, ByVal wParam As Integer, ByVal lParam As Integer) As Integer
    'End Function

    '<DllImport("kernel32.dll", SetLastError:=True)> _
    'Function WriteProfileString(ByVal lpszSection As String, ByVal lpszKeyName As String, ByVal lpszString As String) As Integer
    'End Function

    'Private Sub InstallFont(ByVal FontFile As String)
    '    CopyFontToSystemFonFolder(FontFile)
    '    Dim FontDir As String = IO.Path.GetDirectoryName(FontFile)
    '    Dim FontFilename As String = IO.Path.GetFileName(FontFile)
    '    Dim Fontname As String = IO.Path.GetFileNameWithoutExtension(FontFile)
    '    Dim FontPath As String = FontDir & "\" & FontFilename
    '    Const WM_FONTCHANGE As Integer = &H1D
    '    Const HWND_BROADCAST As Integer = &HFFFF
    '    AddFontResource(FontPath)
    '    SendMessage(HWND_BROADCAST, WM_FONTCHANGE, 0, 0)
    '    WriteProfileString("fonts", FontName & " (TrueType)", FontFileName)
    'End Sub

End Module
