﻿Module GetMouseClickedWord

    ' =============================================================================================
    '  GET CLICKED WORD
    ' =============================================================================================
    Friend Function GetClickedWord(ByVal ctrl As Control, ByVal location As Point) As String
        Dim row As String = GetClickedRow(ctrl, location)
        For i As Int32 = 0 To row.Length - 1
            If TextRenderer.MeasureText(row.Substring(0, i), ctrl.Font).Width > location.X Then
                Return GetCompleteWord(row.Replace("/", " "), i).Trim
            End If
        Next
        Return ""
    End Function

    Private Function GetCompleteWord(ByVal str As String, ByVal i As Int32) As String
        Const separators As String = " " + vbCr + vbLf
        Dim i1, i2 As Int32
        For i1 = i - 1 To 0 Step -1
            If InStr(separators, str(i1)) > 0 Then
                Exit For
            End If
        Next
        For i2 = i To str.Length - 1
            If InStr(separators, str(i2)) > 0 Then
                Exit For
            End If
        Next
        If i1 < 0 Then
            Return str.Substring(0, i2)
        End If
        Return str.Substring(i1, i2 - i1)
    End Function


    ' =============================================================================================
    '  GET CLICKED ROW
    ' =============================================================================================
    Friend Function GetClickedRow(ByVal ctrl As Control, ByVal location As Point) As String
        ' ---------------------------------------------------- split
        Dim sa() As String = SplitTextToRows(ctrl)
        If sa.Length = 0 Then Return ""
        Dim h As Int32 = TextRenderer.MeasureText("123ABC|yç§£%", ctrl.Font).Height
        ' ------------------------------------------------------------------------------
        ' This part is valid for Labels with ContentAlignment.MiddleLeft
        ' TextBox are not implemented because they do not have the vertical alignment
        ' ------------------------------------------------------------------------------
        Dim FirstRow_Y As Int32 = (ctrl.ClientSize.Height - sa.Length * h) \ 2
        If FirstRow_Y < 0 Then FirstRow_Y = 0
        ' ------------------------------------------------------------------------------
        Dim nRow As Int32 = (location.Y - FirstRow_Y) \ h
        If nRow < 0 Then nRow = 0
        If nRow > sa.Length - 1 Then nRow = sa.Length - 1
        Return sa(nRow)
    End Function

    ' =============================================================================================
    '  GET TOTAL TEXT HEIGHT
    ' =============================================================================================
    Friend Function GetTotalTextHeight(ByVal ctrl As Control) As Int32
        Dim nrows As Int32 = SplitTextToRows(ctrl).Length
        Dim h As Int32 = TextRenderer.MeasureText("123ABC|yç§£%", ctrl.Font).Height
        Return nrows * h
    End Function

    ' =============================================================================================
    '  SPLIT TEXT TO ROWS
    ' =============================================================================================
    Friend Function SplitTextToRows(ByVal ctrl As Control) As String()
        Return SplitTextToRows(ctrl.Text, ctrl.Font, ctrl.ClientSize.Width)
    End Function

    ' VERSION THAT -NOT- WORKS WITH WORDS LONGEST THAN THE WINDOW 
    'Friend Function SplitTextToRows(ByVal txt As String, ByVal f As Font, ByVal width As Int32) As String()
    '    Dim ValidString As String
    '    Dim newString As String
    '    Dim newIndex As Int32
    '    Dim newWidth As Int32
    '    Dim ValidIndex As Int32 = 0
    '    Dim sa(-1) As String
    '    Do
    '        ' ------------------------------------------------------------- get valid string
    '        ValidString = txt.Substring(0, ValidIndex)
    '        ' ------------------------------------------------------------- find next space
    '        newIndex = txt.IndexOf(" "c, ValidIndex + 1)
    '        ' ------------------------------------------------------------- 
    '        If newIndex > 0 Then
    '            ' ------------------------------------------------------------- prepare the new width
    '            newString = txt.Substring(0, newIndex)
    '            newWidth = TextRenderer.MeasureText(newString.TrimStart, f).Width
    '            ' ------------------------------------------------------------- test the new width
    '            If newWidth > width Then
    '                ' --------------------------------------------------------- set the new array line
    '                ReDim Preserve sa(sa.Length)
    '                sa(sa.Length - 1) = ValidString.Trim
    '                ' --------------------------------------------------------- remove the line from txt
    '                txt = txt.Substring(ValidIndex + 1)
    '                ValidIndex = 0
    '            Else
    '                ValidIndex = newIndex
    '            End If
    '        Else
    '            ' ------------------------------------------------------------- prepare the new width
    '            newWidth = TextRenderer.MeasureText(txt.TrimStart, f).Width
    '            If newWidth > width Then
    '                ' --------------------------------------------------------- set the new array line
    '                ReDim Preserve sa(sa.Length)
    '                sa(sa.Length - 1) = ValidString
    '                txt = txt.Substring(ValidIndex + 1)
    '                ' --------------------------------------------------------- set the final array line
    '                ReDim Preserve sa(sa.Length)
    '                sa(sa.Length - 1) = txt
    '            Else
    '                ' --------------------------------------------------------- set the new array line
    '                ReDim Preserve sa(sa.Length)
    '                sa(sa.Length - 1) = txt
    '            End If
    '            Exit Do
    '        End If
    '    Loop
    '    ' ----------------------------------------------------
    '    Return sa
    'End Function

    ' VERSION THAT WORKS ALSO WITH WORDS LONGEST THAN THE WINDOW 
    Friend Function SplitTextToRows(ByVal txt As String, ByVal f As Font, ByVal width As Int32) As String()
        Dim ValidString As String
        Dim newString As String
        Dim newIndex As Int32
        Dim newWidth As Int32
        Dim ValidIndex As Int32 = 0
        Dim sa(-1) As String
        If txt = "" Then Return sa
        Do
            ' ------------------------------------------------------------- get valid string
            ValidString = txt.Substring(0, ValidIndex)
            ' ------------------------------------------------------------- find next space
            newIndex = txt.IndexOf(" "c, ValidIndex + 1)
            ' ------------------------------------------------------------- 
            If newIndex > 0 Then
                ' ------------------------------------------------------------- prepare the new width
                newString = txt.Substring(0, newIndex)
                newWidth = TextRenderer.MeasureText(newString.TrimStart, f).Width
                ' ------------------------------------------------------------- test the new width
                If newWidth > width Then
                    ' --------------------------------------------------------- if the new string is a single word
                    If Not newString.Trim.Contains(" ") Then
                        ' --------------------------------------------------------- set the new array line
                        ReDim Preserve sa(sa.Length)
                        sa(sa.Length - 1) = newString.Trim
                        ' --------------------------------------------------------- remove the line from txt
                        txt = txt.Substring(newString.Length + 1)
                        ValidIndex = 0
                    Else
                        ' --------------------------------------------------------- set the new array line
                        ReDim Preserve sa(sa.Length)
                        sa(sa.Length - 1) = ValidString.Trim
                        ' --------------------------------------------------------- remove the line from txt
                        txt = txt.Substring(ValidIndex + 1)
                        ValidIndex = 0
                    End If
                Else
                    ValidIndex = newIndex
                End If
            Else
                ' ------------------------------------------------------------- prepare the new width
                newWidth = TextRenderer.MeasureText(txt.TrimStart, f).Width
                If newWidth > width Then
                    ' --------------------------------------------------------- set the new array line
                    ReDim Preserve sa(sa.Length)
                    sa(sa.Length - 1) = ValidString
                    txt = txt.Substring(ValidIndex + 1)
                    ' --------------------------------------------------------- set the final array line
                    ReDim Preserve sa(sa.Length)
                    sa(sa.Length - 1) = txt
                Else
                    ' --------------------------------------------------------- set the new array line
                    ReDim Preserve sa(sa.Length)
                    sa(sa.Length - 1) = txt
                End If
                Exit Do
            End If
        Loop
        ' ----------------------------------------------------
        Return sa
    End Function

End Module
