﻿Module Module_Utils

    Friend GCI As Globalization.CultureInfo = Globalization.CultureInfo.InvariantCulture

    Friend RTBLines() As String


    ' =============================================================================================
    '  WINDOWS SCHEDULER PRECISION
    ' ============================================================================================= 
    Private Declare Function timeBeginPeriod Lib "winmm.dll" (ByVal uPeriod As Int32) As Int32
    Private Declare Function timeEndPeriod Lib "winmm.dll" (ByVal uPeriod As Int32) As Int32
    Friend Sub SchedulerMaxPrecision()
        If OperatingSystemIsWindows Then
            timeBeginPeriod(1)
        End If
    End Sub
    Friend Sub ShedulerDefaultPrecision()
        If OperatingSystemIsWindows Then
            timeEndPeriod(1)
        End If
    End Sub

    ' ----------------------------------------------------------------------------------
    '  StartTime STOP WATCH
    ' ----------------------------------------------------------------------------------
    Friend StartTime As Stopwatch = New Stopwatch
    Friend Sub SetStartTime()
        StartTime.Reset()
        StartTime.Start()
    End Sub

    ' ----------------------------------------------------------------------------------
    '  Swap
    ' ----------------------------------------------------------------------------------
    Friend Sub Swap(Of T)(ByRef d1 As T, ByRef d2 As T)
        Dim d As T = d2
        d2 = d1
        d1 = d
    End Sub

    ' ----------------------------------------------------------------------------------
    '  Extract from "text" the complete word pointed by the "selectionIndex"
    ' ----------------------------------------------------------------------------------
    Friend Function GetCompleteWord(ByVal text As String, _
                                    ByVal selectionIndex As Int32) As String
        Dim start, length As Int32
        FindWordPointers(text, selectionIndex, start, length)
        Return text.Substring(start, length)
    End Function

    ' ----------------------------------------------------------------------------------------------
    '  Extract from "text" the complete word pointed by the "selectionIndex" (short words version)
    ' ----------------------------------------------------------------------------------------------
    Friend Function GetCompleteWord_Short(ByVal text As String, _
                                          ByVal selectionIndex As Int32) As String
        Dim start, length As Int32
        FindWordPointers_ShortWords(text, selectionIndex, start, length)
        Return text.Substring(start, length)
    End Function

    ' ----------------------------------------------------------------------------------
    '  Extract from "text" start and length of the word pointed by the "selectionIndex"
    ' ----------------------------------------------------------------------------------
    Friend Sub FindWordPointers(ByVal text As String, _
                                ByVal selectionIndex As Int32, _
                                ByRef start As Int32, _
                                ByRef lenght As Int32)
        Const separators As String = " " + vbCr + vbLf + vbTab
        Dim i1, i2 As Int32
        For i1 = selectionIndex To 1 Step -1
            If InStr(separators, text(i1 - 1)) > 0 Then
                Exit For
            End If
        Next
        For i2 = selectionIndex To text.Length - 1
            If InStr(separators, text(i2)) > 0 Then
                Exit For
            End If
        Next
        start = i1
        lenght = i2 - i1
    End Sub

    ' ----------------------------------------------------------------------------------
    '  Extract from "text" start and length of the word pointed by the "selectionIndex"
    '  This version find shortest words - not including characters like  " , / () ...
    ' ----------------------------------------------------------------------------------
    Friend Sub FindWordPointers_ShortWords(ByVal text As String, _
                                            ByVal selectionIndex As Int32, _
                                            ByRef start As Int32, _
                                            ByRef lenght As Int32)
        '
        Const separators As String = " "";:,/\|()'" + vbCr + vbLf + vbTab
        '
        Dim i1, i2 As Int32
        For i1 = selectionIndex To 1 Step -1
            If InStr(separators, text(i1 - 1)) > 0 Then
                Exit For
            End If
        Next
        For i2 = selectionIndex To text.Length - 1
            If InStr(separators, text(i2)) > 0 Then
                Exit For
            End If
        Next
        start = i1
        lenght = i2 - i1
    End Sub

    ' ----------------------------------------------------------------------------------
    '  Extract from "text" start and length of the word pointed by the "selectionIndex"
    '  This version find a complete string starting and finishing with double quote
    ' ----------------------------------------------------------------------------------
    Friend Sub FindQuotedStringPointers(ByVal text As String, _
                            ByVal selectionIndex As Int32, _
                            ByRef start As Int32, _
                            ByRef lenght As Int32)
        '
        Const separators As String = ";:,/\|()'" + vbCr + vbLf
        '
        Dim i1, i2 As Int32
        For i1 = selectionIndex To 1 Step -1
            If InStr(separators, text(i1 - 1)) > 0 Then
                lenght = 0
                Return
            End If
            If text(i1 - 1) = """" Then
                Exit For
            End If
        Next
        For i2 = selectionIndex To text.Length - 1
            If InStr(separators, text(i2)) > 0 Then
                lenght = 0
                Return
            End If
            If text(i2) = """" Then
                Exit For
            End If
        Next
        start = i1
        lenght = i2 - i1
    End Sub



    ' ----------------------------------------------------------------------------------
    '  Extract numeric params from a string
    '  Valid only if fields are not more than MaxFieldNumber 
    ' ----------------------------------------------------------------------------------
    Friend Function ExtractNumericParams(ByVal s As String, ByVal MaxFieldsNumber As Int32) As Double()
        Dim n(-1) As Double
        Dim sa As String() = s.Split(" "c)
        Dim paramsCount As Int32 = 0
        For i As Int32 = 0 To sa.Length - 1
            If sa(i) <> "" Then
                If IsValidDouble(sa(i)) Then
                    ReDim Preserve n(n.Length)
                    n(n.Length - 1) = EvalDouble(sa(i))
                End If
                paramsCount += 1
            End If
        Next
        If paramsCount > MaxFieldsNumber Then ReDim n(-1)
        Return n
    End Function

    ' =======================================================================================
    '  TrayWnd
    ' =======================================================================================
    'Private Declare Function FindWindow Lib "user32" Alias "FindWindowA" (ByVal lpClassName As String, ByVal lpWindowName As String) As Int32
    'Private Declare Function ShowWindow Lib "user32" (ByVal hwnd As Int32, ByVal nCmdShow As Int32) As Int32
    'Private Const SW_HIDE As Int32 = 0
    'Private Const SW_SHOW As Int32 = 5

    'Friend Sub HideTrayWnd()
    '    Dim lngHandle As Int32
    '    lngHandle = FindWindow("Shell_TrayWnd", "")
    '    ShowWindow(lngHandle, SW_HIDE)
    'End Sub

    'Friend Sub ShowTrayWnd()
    '    Dim lngHandle As Int32
    '    lngHandle = FindWindow("Shell_TrayWnd", "")
    '    ShowWindow(lngHandle, SW_SHOW)
    'End Sub

    ' =============================================================================
    '   START PROCESS (IF NOT RUNNING) AND WAIT
    ' =============================================================================
    Friend Sub StartProcess_IfNotRunning_AndWait(ByVal filename As String)
        ' ------------------------------------------------------------------ Prepare path
        If Not IO.Path.IsPathRooted(filename) Then
            filename = Application.StartupPath + "\" + filename
        End If
        If Not IO.File.Exists(filename) Then Return
        ' ------------------------------------------------------------------ Test if already running
        If AppIsRunning(filename) Then Return
        ' ------------------------------------------------------------------ Init process
        Dim proc As Process
        Dim psi As New ProcessStartInfo
        psi.WorkingDirectory = IO.Path.GetDirectoryName(filename)
        psi.FileName = filename
        ' ------------------------------------------------------------------ protect for process errors
        Try
            ' -------------------------------------------------------------- Start process
            proc = Process.Start(psi)
            ' -------------------------------------------------------------- Wait process really started 
            For i As Int32 = 1 To 100
                Thread.Sleep(100)
                proc.Refresh()
                If proc.MainWindowTitle.Contains(".") Then Exit For
            Next
        Catch
        End Try
        Thread.Sleep(500)
    End Sub

    ' =============================================================================
    '   App Is Running  (testing the complete path of the running app)
    ' =============================================================================
    Friend Function AppIsRunning(ByVal FileName As String) As Boolean
        ' ------------------------------------------------------------------ Correct path for /../..
        FileName = IO.Path.GetFullPath(FileName)
        ' ------------------------------------------------------------------ Normalize filename
        FileName = FileName.Replace("/", "\")
        ' ------------------------------------------------------------------ Compare with all processes
        Dim p() As System.Diagnostics.Process
        p = System.Diagnostics.Process.GetProcesses()
        For i As Int32 = 0 To p.Length - 1
            Try
                If Not p(i).ProcessName.ToLower.StartsWith("theremino_") Then Continue For ' <-- fast execution

                ' REMOVED 31/01/2020
                'If p(i).MainWindowTitle = "" Then Continue For

                If p(i).MainModule.FileName = FileName Then Return True
            Catch
            End Try
        Next
        Return False
    End Function

    ' ==============================================================================================================
    '   Close All ThereminoApps
    ' ==============================================================================================================
    Friend Sub CloseAllThereminoApps()
        Dim p() As System.Diagnostics.Process
        p = System.Diagnostics.Process.GetProcesses()
        For i As Int32 = 0 To p.Length - 1
            If Not p(i).ProcessName.StartsWith("Theremino_", StringComparison.OrdinalIgnoreCase) Then Continue For
            If p(i).Id = System.Diagnostics.Process.GetCurrentProcess.Id Then Continue For
            '
            'Debug.Print(p(i).ProcessName)
            '
            p(i).CloseMainWindow()
            p(i).Close()
        Next
        '
        p = System.Diagnostics.Process.GetProcesses()
        For i As Int32 = 0 To p.Length - 1
            If Not p(i).ProcessName.StartsWith("Theremino_", StringComparison.OrdinalIgnoreCase) Then Continue For
            If p(i).Id = System.Diagnostics.Process.GetCurrentProcess.Id Then Continue For
            '
            'Debug.Print(p(i).ProcessName)
            '
            If Not p(i).WaitForExit(1000) Then
                Try
                    p(i).Kill()
                Catch ex As Exception
                End Try
            End If
        Next
    End Sub

    ' ==============================================================
    '  REPLACE MULTIPLE SPACES and TABS and TRIM
    '  (3..5 uS for Strings with 32 chars)
    ' ============================================================== 
    Friend Function ReplaceMultipleSpacesAndTabAndTrim(ByVal s As String) As String
        s = s.Replace(vbTab, " ")
        While s.Contains("  ")
            s = s.Replace("  ", " ")
        End While
        Return s.Trim
    End Function

    'Friend Function ReplaceMultipleSpacesAndTrim_PreserveQuotedSpaces(ByVal s As String) As String
    '    ' ------------------------------------------------- replace inside quotes spaces with chr(255)
    '    Dim InsideQuotes As Boolean = False
    '    Dim s2 As String = ""
    '    For i As Int32 = 0 To s.Length - 1
    '        If s(i) = """" Then InsideQuotes = Not InsideQuotes
    '        If InsideQuotes Then
    '            If s(i) = " " Then
    '                s2 += Chr(160)   'NBSP
    '            Else
    '                s2 += s(i)
    '            End If
    '        Else
    '            s2 += s(i)
    '        End If
    '    Next
    '    ' ------------------------------------------------- remove all double spaces
    '    While s2.Contains("  ")
    '        s2 = s2.Replace("  ", " ")
    '    End While
    '    ' ------------------------------------------------- replace inside quotes chr(255) with spaces
    '    's2 = s2.Replace(Chr(160), " ")
    '    ' ------------------------------------------------- trim
    '    Return s2.Trim
    'End Function

    Friend Function ReplaceMultipleSeparators(ByVal s As String, ByVal separator As String) As String
        If separator = " " Then
            Return ReplaceMultipleSpacesAndTabAndTrim(s)
        Else
            While s.Contains(separator + separator)
                s = s.Replace(separator + separator, separator)
            End While
            Return s.Trim
        End If
    End Function


    ' =============================================================================
    '   GetRightPartAfterThisWord
    ' =============================================================================
    Friend Function GetRightPartAfterThisLowerCaseWord(ByVal str As String, ByVal word As String) As String
        Dim i As Int32 = str.ToLower.IndexOf(word) + word.Length
        If i > 1 And i < str.Length Then
            Return str.Remove(0, i)
        Else
            Return ""
        End If
    End Function



    ' ======================================================================================================
    '  ASYNC KEY STATE
    ' ======================================================================================================

    Private Declare Function GetAsyncKeyState Lib "user32" (ByVal vKey As Int32) As Int16

    Private Declare Function VkKeyScan Lib "user32" Alias "VkKeyScanA" (ByVal chr As Int16) As Int16

    Private Const PRESSED_NOW As Int32 = &H8000
    Private Const PRESSED_AFTER_PREVIOUS_CALL As Int32 = &H1

    Friend Function KeyPressedNow(ByVal k As Int32) As Boolean
        KeyPressedNow = (GetAsyncKeyState(k) And PRESSED_NOW) <> 0
    End Function

    Friend Function KeyPrevious(ByVal k As Int32) As Boolean
        KeyPrevious = (GetAsyncKeyState(k) And PRESSED_AFTER_PREVIOUS_CALL) <> 0
    End Function

    Friend Function KeyCodeToVirtualKey(ByVal k As Int32) As Int16
        If k > 65535 Then k = 0
        Return VkKeyScan(CShort(k))
    End Function

    Friend Sub WaitMouseOff()
        While KeyPressedNow(Keys.LButton) Or KeyPressedNow(Keys.MButton) Or KeyPressedNow(Keys.RButton)
            Thread.Sleep(1)
        End While
    End Sub

    Friend Sub WaitKeyOff(ByVal WaitKey As Int32)
        Do
            Thread.Sleep(1)
        Loop Until Not KeyPressedNow(WaitKey)
    End Sub

    Friend Function GetKeyCode(ByVal str As String) As Keys
        Select Case str.ToLower
            Case "1" : Return Keys.D1
            Case "2" : Return Keys.D2
            Case "3" : Return Keys.D3
            Case "4" : Return Keys.D4
            Case "5" : Return Keys.D5
            Case "6" : Return Keys.D6
            Case "7" : Return Keys.D7
            Case "8" : Return Keys.D8
            Case "9" : Return Keys.D9
            Case "0" : Return Keys.D0
            Case "a" : Return Keys.A
            Case "b" : Return Keys.B
            Case "c" : Return Keys.C
            Case "d" : Return Keys.D
            Case "e" : Return Keys.E
            Case "f" : Return Keys.F
            Case "g" : Return Keys.G
            Case "h" : Return Keys.H
            Case "i" : Return Keys.I
            Case "j" : Return Keys.J
            Case "k" : Return Keys.K
            Case "l" : Return Keys.L
            Case "m" : Return Keys.M
            Case "n" : Return Keys.N
            Case "o" : Return Keys.O
            Case "p" : Return Keys.P
            Case "q" : Return Keys.Q
            Case "r" : Return Keys.R
            Case "s" : Return Keys.S
            Case "t" : Return Keys.T
            Case "u" : Return Keys.U
            Case "v" : Return Keys.V
            Case "w" : Return Keys.W
            Case "x" : Return Keys.X
            Case "y" : Return Keys.Y
            Case "z" : Return Keys.Z
            Case "f1" : Return Keys.F1
            Case "f2" : Return Keys.F2
            Case "f3" : Return Keys.F3
            Case "f4" : Return Keys.F4
            Case "f5" : Return Keys.F5
            Case "f6" : Return Keys.F6
            Case "f7" : Return Keys.F7
            Case "f8" : Return Keys.F8
            Case "f9" : Return Keys.F9
            Case "f10" : Return Keys.F10
            Case "f11" : Return Keys.F11
            Case "f12" : Return Keys.F12
            Case "left" : Return Keys.Left
            Case "right" : Return Keys.Right
            Case "up" : Return Keys.Up
            Case "down" : Return Keys.Down
            Case "space" : Return Keys.Space
            Case "esc" : Return Keys.Escape
            Case "pageup" : Return Keys.PageUp
            Case "pagedown" : Return Keys.PageDown
            Case "home" : Return Keys.Home
            Case "end" : Return Keys.End
            Case "shift" : Return Keys.ShiftKey
            Case "ctrl" : Return Keys.ControlKey
            Case "alt" : Return Keys.Menu
        End Select
        Return Keys.None
    End Function


    ' ==============================================================
    '  RICH TEXT BOX - SUPPRESS SOUNDS (call from KeyDown)
    ' ==============================================================
    Friend Sub RichTextBox_SuppressSounds(ByVal sender As Object, ByRef e As KeyEventArgs)
        Dim rtb As RichTextBox = CType(sender, RichTextBox)
        If rtb.GetLineFromCharIndex(rtb.SelectionStart) = 0 AndAlso _
                                            (e.KeyData = Keys.Up Or e.KeyData = Keys.PageUp) _
        OrElse rtb.GetLineFromCharIndex(rtb.SelectionStart) = rtb.GetLineFromCharIndex(rtb.TextLength) AndAlso _
                                            (e.KeyData = Keys.Down Or e.KeyData = Keys.PageDown) _
        OrElse rtb.SelectionStart = rtb.TextLength AndAlso e.KeyData = Keys.Right _
        OrElse rtb.SelectionStart = 0 AndAlso _
                (e.KeyData = Keys.Left Or (e.KeyData = Keys.Back And rtb.SelectionLength = 0)) Then
            e.Handled = True
        End If
    End Sub


    ' ======================================================================================================
    '  DEBUG FUNCTIONS (Debug window)
    ' ======================================================================================================
    Private DebugStopWatch As Stopwatch = New Stopwatch
    Friend Sub Debug_InitTime()
        DebugStopWatch.Reset()
        DebugStopWatch.Start()
    End Sub
    Friend Sub Debug_Print(ByVal s As String)
        Debug.Print(DebugStopWatch.Elapsed.TotalSeconds.ToString("0.000", GCI).PadLeft(8) + " " + s)
    End Sub
    Friend Sub Debug_PrintMicro(ByVal s As String)
        Debug.Print(DebugStopWatch.Elapsed.TotalSeconds.ToString("0.000000", GCI).PadLeft(8) + " " + s)
    End Sub
    Friend Sub Debug_PrintDiffTime(ByVal s As String)
        Debug.Print(DebugStopWatch.Elapsed.TotalSeconds.ToString("0.000", GCI).PadLeft(8) + " " + s)
        Debug_InitTime()
    End Sub

    ' ======================================================================================================
    '  DEBUG FUNCTIONS (Form1 text)
    ' ======================================================================================================
    'Private DebugStopWatch As Stopwatch = New Stopwatch
    'Friend Sub Debug_InitTime()
    '    DebugStopWatch.Reset()
    '    DebugStopWatch.Start()
    'End Sub
    'Friend Sub Debug_Print(ByVal s As String)
    '    Form1.Text = DebugStopWatch.Elapsed.TotalSeconds.ToString("0.000", GCI).PadLeft(8) + " " + s
    'End Sub
    'Friend Sub Debug_PrintMicro(ByVal s As String)
    '    Form1.Text = DebugStopWatch.Elapsed.TotalSeconds.ToString("0.000000", GCI).PadLeft(8) + " " + s
    'End Sub
    'Friend Sub Debug_PrintDiffTime(ByVal s As String)
    '    Form1.Text = DebugStopWatch.Elapsed.TotalSeconds.ToString("0.000", GCI).PadLeft(8) + " " + s
    '    Debug_InitTime()
    'End Sub


    ' ======================================================================================================
    '  OVERWRITE MODE CARET
    ' ======================================================================================================
    Private Declare Function CreateCaret Lib "user32" (ByVal hWnd As IntPtr, _
                                                      ByVal hBitmap As IntPtr, _
                                                      ByVal nWidth As Int32, _
                                                      ByVal nHeight As Int32) As Int32

    Private Declare Function ShowCaret Lib "user32" (ByVal hWnd As IntPtr) As Int32

    Private OverwriteMode As Boolean

    Friend Sub OverwriteMode_Toggle(ByRef rtb As Theremino_Automation.SyntaxRichTextBox)
        OverwriteMode = Not OverwriteMode
        If Not OverwriteMode Then
            CreateCaret(rtb.Handle, IntPtr.Zero, 1, CInt((rtb.Font.Height - 1) * rtb.ZoomFactor))
            ShowCaret(rtb.Handle)
        End If
    End Sub

    Friend Sub OverwriteMode_ShowCaret(ByRef rtb As Theremino_Automation.SyntaxRichTextBox)
        If OverwriteMode Then
            CreateCaret(rtb.Handle, IntPtr.Zero, CInt(12 * rtb.ZoomFactor), CInt((rtb.Font.Height - 1) * rtb.ZoomFactor))
            ShowCaret(rtb.Handle)
        End If
    End Sub


    ' ======================================================================================================
    '  STRING SPACED AROUND OPERATORS 
    ' ======================================================================================================
    Friend Function StringSpacedAroundOperators(ByVal s As String) As String
        s = SpacesAroundOperators(s)
        s = ReplaceMultipleSpacesAndTabAndTrim(s)
        Return " " + s + " "
    End Function

    Private SubtractionChars As String = ChrW(&H2D) + " " + _
                                         ChrW(&H2010) + " " + _
                                         ChrW(&H2212) + " " + _
                                         ChrW(&H2013) + " " + _
                                         ChrW(&H2014)

    Dim Operators As String = SubtractionChars + " ( ) * / + = ^ ,"

    Dim OperatorsArray() As String = Operators.Replace("  ", " ").Split(" "c)

    Private Function SpacesAroundOperators(ByVal s As String) As String
        If s = "" Then Return s
        For Each Str As String In OperatorsArray
            If Str <> "" Then
                s = s.Replace(Str, " " + Str + " ")
            End If
        Next
        Return s
    End Function


    ' ======================================================================================================
    '  MOUSE POSITION NORMALIZED AND LIMITED TO THE SCREEN CONTAINING THE FORM
    ' ------------------------------------------------------------------------------------------------------
    '  To call from another Thread, the Form1_Location is updated in the Form_LocationChanged event 
    ' ======================================================================================================
    Friend Form1_Location As Point

    Friend Function GetNormalizedCursorX() As Double
        ' ---------------------------------------------------- get the cursor position in screen coordimates
        Dim mousePoint As Point = Control.MousePosition
        ' ---------------------------------------------------- get the screen containing the form
        Dim s As Screen = Screen.FromPoint(Form1_Location)
        ' ---------------------------------------------------- normalize X position to the screen
        Dim w As Int32 = s.Bounds.Width - 1
        Dim x As Double = (mousePoint.X - s.Bounds.X) / w
        ' ---------------------------------------------------- limit X in the range 0 to 1
        If x < 0 Then x = 0
        If x > 1 Then x = 1
        Return x
    End Function

    Friend Function GetNormalizedCursorY() As Double
        ' ---------------------------------------------------- get the cursor position in screen coordimates
        Dim mousePoint As Point = Control.MousePosition
        ' ---------------------------------------------------- get the screen containing the form
        Dim s As Screen = Screen.FromPoint(Form1_Location)
        ' ---------------------------------------------------- normalize Y position to the screen
        Dim h As Int32 = s.Bounds.Height - 1
        Dim y As Double = (mousePoint.Y - s.Bounds.Y) / h
        ' ---------------------------------------------------- limit Y in the range 0 to 1
        If y < 0 Then y = 0
        If y > 1 Then y = 1
        If Control.ModifierKeys = Keys.Control Then
            Beep()
        End If
        Return 1 - y
    End Function


    ' ======================================================================================================
    '  COLORS
    ' ======================================================================================================
    Private Function StringStartingWithLetter(ByVal str As String) As Boolean
        Return Char.IsLetter(str(0))
    End Function

    Friend Function GrayScaleLuminance(ByVal c As Color) As Byte
        Dim lum As Double = 0.299 * c.R + 0.587 * c.G + 0.114 * c.B
        Return CByte(lum)
    End Function

    Friend Function ModifiedColor(ByVal c As Color, ByVal delta As Int32) As Color
        Dim r As Int32 = c.R + delta
        Dim g As Int32 = c.G + delta
        Dim b As Int32 = c.B + delta
        If r < 0 Then r = 0
        If g < 0 Then g = 0
        If b < 0 Then b = 0
        If r > 255 Then r = 255
        If g > 255 Then g = 255
        If b > 255 Then b = 255
        Return Color.FromArgb(r, g, b)
    End Function

    Friend Function GetColorNamesAsArray() As String()
        Dim sa() As String = GetColorNames().Split(" "c)
        Return sa
    End Function

    Friend Function GetColorNames() As String
        GetColorNames = "Black MidnightBlue Navy DarkBlue MediumBlue Blue DarkSlateGray DarkSlateBlue " + _
               "Indigo Maroon DarkRed Purple DarkMagenta BlueViolet DarkOrchid DarkViolet " + _
               "MediumOrchid MediumPurple MediumSlateBlue SlateBlue RoyalBlue DodgerBlue " + _
               "DeepSkyBlue CornflowerBlue SteelBlue CadetBlue LightSeaGreen DarkCyan Teal " + _
               "DarkGreen Green ForestGreen SeaGreen DarkOliveGreen Olive OliveDrab LimeGreen " + _
               "MediumSeaGreen Sienna SaddleBrown Brown Firebrick Crimson Red MediumVioletRed " + _
               "DeepPink Fuchsia Magenta IndianRed OrangeRed Tomato Chocolate DarkGoldenrod " + _
               "DimGray Gray SlateGray LightSlateGray DarkGray Silver LightSteelBlue LightBlue " + _
               "DarkSeaGreen MediumAquamarine DarkTurquoise MediumTurquoise Peru Goldenrod DarkKhaki " + _
               "RosyBrown PaleVioletRed Orchid Violet Plum HotPink LightCoral Salmon DarkSalmon Coral " + _
               "DarkOrange Orange SandyBrown LightSalmon BurlyWood Tan Thistle LightPink Pink " + _
               "Gold Yellow Lime SpringGreen MediumSpringGreen YellowGreen Chartreuse LawnGreen " + _
               "GreenYellow PaleGreen LightGreen LightSkyBlue SkyBlue Turquoise Aqua Cyan " + _
               "Aquamarine PaleTurquoise PowderBlue LightCyan LightGray Gainsboro Lavender Khaki " + _
               "PaleGoldenrod PeachPuff NavajoWhite Wheat Bisque Moccasin MistyRose BlanchedAlmond " + _
               "PapayaWhip AntiqueWhite LavenderBlush SeaShell Linen OldLace Beige " + _
               "Cornsilk LightYellow LightGoldenrodYellow LemonChiffon FloralWhite Ivory Snow " + _
               "Azure AliceBlue Honeydew MintCream WhiteSmoke GhostWhite White"
        GetColorNames += " " + GetColorNames2()
    End Function

    'Friend Function GetColorNames3() As String
    '    Dim s As String = ""
    '    Dim i As Int32
    '    For i = 1 To 9
    '        s += "Red" + i.ToString + " "
    '    Next
    '    For i = 1 To 9
    '        s += "Green" + i.ToString + " "
    '    Next
    '    For i = 1 To 9
    '        s += "Blue" + i.ToString + " "
    '    Next
    '    For i = 1 To 9
    '        s += "Cyan" + i.ToString + " "
    '    Next
    '    For i = 1 To 9
    '        s += "Magenta" + i.ToString + " "
    '    Next
    '    For i = 1 To 9
    '        s += "Yellow" + i.ToString + " "
    '    Next
    '    For i = 1 To 9
    '        s += "Gray" + i.ToString + " "
    '    Next
    '    Return s.Trim
    'End Function

    Friend Function GetColorNames2() As String
        Dim s As String = ""
        Dim i As Int32
        For i = 1 To 9
            s += "Red" + i.ToString + " "
            s += "Green" + i.ToString + " "
            s += "Blue" + i.ToString + " "
            s += "Cyan" + i.ToString + " "
            s += "Magenta" + i.ToString + " "
            s += "Yellow" + i.ToString + " "
            s += "Gray" + i.ToString + " "
        Next
        Return s.Trim
    End Function


    Friend Function ColorFromColorName(ByVal str As String) As Color
        ' -----------------------------------------------
        Dim n As Double = Val(Right(str, 1))
        If n <> 0 Then
            n = n / 9
            n = n ^ 0.3
            n *= 255
            Dim r, g, b As Double
            Select Case Left(str, str.Length - 1).ToLower
                Case "red" : r = n
                Case "green" : g = n
                Case "blue" : b = n
                Case "magenta" : r = n : b = n
                Case "yellow" : r = n : g = n
                Case "cyan" : g = n : b = n
                Case "gray" : r = n : g = n : b = n
            End Select
            Return Color.FromArgb(CInt(r), CInt(g), CInt(b))
        End If
        ' -----------------------------------------------
        Dim c As Color = Color.FromName(str)
        If c.A <> 0 Then
            Return c
        End If
        Return Color.Empty
    End Function

End Module
